//*****************************************************************************
//   donation_create_renewal_form_current_membership - (For use with SalesForce
//      spring 2008 nonprofit template) When executed from a displayed 
//      "donation/opportunity" record of record type "membership", will create
//      a new "donation/opportunity" record representing an expected "renewal"
//      from the same "Contact" and "Organization/Account", and with same 
//      "Contact Roles" as the starting record
//   Copyright (C) 2008  Larry Francis Bednar
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or (at your option) any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//   The author may be contacted at:
//      3226 NE 24th Ave, Portland, OR 97212-2442 USA
//      lb@quimdas.com
//      www.quimdas.com
//
//   $Revision: 1.4 $, $Date: 2008/12/12 18:28:43 $
//****************************************************************************
//NOTES
//  This version of this S-control is a revision of a version created by 
//  Steve Anderson, One/NW.  The Anderson version used the Ajax Toolkit beta
//  V3.3.  This version instead uses the V13.0 Ajax Toolkit.  A number of 
//  additional changes (to commenting, variable naming conventions, etc) 
//  have also been made.  But the basic logic attempts to follow the Anderson
//  version.
//
//****************************************************************************

//Include library of JavaScript date functions
{!INCLUDE($SControl.Functions_For_Dates)}

//Include library of JavaScript functions for checking field presence
{!INCLUDE($SControl.Functions_For_Field_Checks)}

/****************************************************************
* Main function called on page load
*****************************************************************/
function initPage() {				
    //alert("initPage() running");
    var startUpResult = createRenewal();
}

/********************************************************************
* function to create membership renewal record
********************************************************************/
function createRenewal() {
   //alert("createRenewal() running");
   /****************************************************************
   * Constants may differ between salesforce instances
   *****************************************************************/
   //Modified from use of Ajax beta 3.3 toolkit to use of V13.0 toolkit
   //   L.Bednar, 09Oct2008

   //Test whether all expected fields are present in db
   var testFieldNames = ["Anonymous__c", "Fund__c", "Membership_Length__c", 
       "Membership_Start_Date__c", "Membership_Type__c"];
   //alert("testFieldNames: " + testFieldNames.join()); 
   var testTableName = "Opportunity";
   var testFieldResult = missingFields(testTableName, testFieldNames);
   // missingFields function returns a string listing any tested fields that
   //   are not in db, returns empty string if all fields are present
   if ( !(testFieldResult == "") ) {
      alert("The following custom fields are used by this  " + 
            "S-control, but are not present in your database: \n\r" +
            testFieldResult );
   }
   //alert("passed field check");

   //Get vals from starting opportunity
   var oppId = "{!Opportunity.Id}";
   //alert("oppId: " + oppId);
   var acctName = "{!Opportunity.Account}";
   //alert("AccountName: " + acctName);

   var defaultMbrDuration = 12;  //default membership duration, in months
   var defaultMbrType = "Renewal";
   var stage = "Open";    //default stage for renewals associated with active memberships

   // get current opportunity info ************************************************************
   // handling of membership_start_date__c and membership_length__c added 
   //   L.Bednar 14Aug2008 
   var fieldList = "AccountId, Amount, Anonymous__c, " +
                   "CloseDate, Fund__c, LeadSource, " +
                   "Membership_Length__c, Membership_Start_Date__c, " +
                   "Membership_Type__c, " +
                   "Name, RecordTypeId, Type"; 
   //var fieldList = "Amount, Name, Type"; 
   //alert("fieldList: " + fieldList);
   try {
      var oppOld = sforce.connection.retrieve( fieldList, "Opportunity", [oppId])[0];
   }
   catch(error) {
      alert("Error retrieving oppOld: (" + error.faultcode + ") " + error.faultstring );
   }
   //alert("passed opp retrieval");

   //Test whether all required field values are present on starting record
   var testFieldValResult = "";
   //Any field tested for values should have an if block similar to the next one
   if ( !(oppOld.Membership_Start_Date__c) ) {
      testFieldValResult += "Membership_Start_Date__c";
   }
   //If tests for any required values indicated problems, print message for user
   if ( !(testFieldValResult == "") ) {
      alert("The following REQUIRED fields do NOT have values: \n\r" + 
            testFieldValResult ); 
   }
   //alert("passed value check");

   // Membership duration must be retrieved before close and membership date calcs
   // If no duration specified, set to default value
   var oppMbrDuration = oppOld.Membership_Length__c;
   if ( !(oppMbrDuration) ) {
      var mbrDuration = defaultMbrDuration;
      alert("No membership duration specified.  Default value of " + defaultMbrDuration + " months used");
   }
   //alert("mbrDuration: " + mbrDuration);

   // calculate new membership start date
   var mbrStartDate = dateAddMonths( oppOld.getDate("Membership_Start_Date__c"), mbrDuration );
   //alert("recalculated mbrStartDate:" + mbrStartDate);

   // calculate new estimated close date
   // expected close date will be the date on which the next paid membership period starts
   //   close date for current rec is inappropriate because pmt for membership may arrive
   //   earlier or later than the actual start date of the new membership period
   var closeDate = mbrStartDate;
   // getMonth returns month in (0,11) basis, convert to (1,12) basis for use in record name
   var closeDateLabel = dateToTextMDY( mbrStartDate ); 
   //alert("closeDateLabel: " + closeDateLabel );

   // Query salesforce for the contact roles on the selected opportunity ***********************

   var rolesOldQryStmt = "SELECT ContactId, Role, IsPrimary " + 
                           "FROM OpportunityContactRole " +
                          "WHERE OpportunityId = '" + oppId + "'"
   var rolesOld = sforce.connection.query( rolesOldQryStmt );
   //alert("passed rolesOld query");
   // put the response record in an array
   //var rolesOldArray = rolesOld.records;
   var rolesOldArray = rolesOld.getArray("records");
   //alert("rolesOldArray.length: " + rolesOldArray.length );
   for( row = 0; row < rolesOldArray.length; row++ ) {

      var thisRole = rolesOldArray[row];
      //alert("one contact role row visited");

      // set values from the 'primary' contact role
      if (thisRole.getBoolean("IsPrimary")) {
         var contactId = thisRole.ContactId;
         var roleName = thisRole.Role;
      }
   }	

   // retrieve primary contact info
   var primaryContact = sforce.connection.retrieve("FirstName, LastName","Contact", [contactId])[0]; 

   // if the primary role is individual donor, set the naming prefix to the Contact name, 
   //   otherwise to the linked Account
   if (roleName == "Individual Donor") {
      var namePrefix = primaryContact.FirstName + " " + primaryContact.LastName;
   } else {
      var namePrefix = acctName;
   }
   //alert("Opportunity Name Prefix: " + namePrefix);

   // create object for new opportunity ********************************************************
   var oppNew = new sforce.SObject("Opportunity");
   var oppNewArray = new Array(1);

   oppNew.AccountId = oppOld.AccountId
   oppNew.Amount = oppOld.Amount
   oppNew.Anonymous__c = oppOld.Anonymous__c;
   oppNew.CloseDate =  closeDate;
   oppNew.Fund__c = oppOld.Fund__c;
   oppNew.LeadSource = oppOld.LeadSource;

   //NOTE: Merge field used to obtain value here - ALSO extracted with query of starting
   //   opp rec.  Might be wise to use just ONE technique within this function. 
   var recTypeTxt="{!Opportunity.RecordType}";
   // set the matching percent - NOTE type "match" not used in NWEI as of 25Aug2008
   //   and this record type should anyway NOT be a starting point for this automation
   if (recTypeTxt=="match") {
      oppNew.Matching__c = 100;
   }
   //alert("Opp.RecordType merge val: " + recTypeTxt +  "\n"
   //       + "oppOld.Type: " + oppOld.Type);
    
   oppNew.Membership_Length__c = oppMbrDuration;
   oppNew.Membership_Start_Date__c = mbrStartDate;
   oppNew.Membership_Type__c = defaultMbrType;

   var nameSuffix = "";
   if ( recTypeTxt == "Membership" ) {
      nameSuffix = 'Membership';
   } else {
      nameSuffix = "NonMembership";  //Should never arise
   }
   var newName = namePrefix + " " + nameSuffix + " - " + closeDateLabel;
   oppNew.Name = newName;
   //alert("oppNew.Name: " + newName);

   oppNew.RecordTypeId = oppOld.RecordTypeId;
   oppNew.StageName = stage;
   oppNew.Type =  oppOld.Type;
   //alert("oppNew.Type: " + oppNew.Type );

   // Store object in array
   var oppNewArray = new Array();
   oppNewArray[0] = oppNew;
   //alert("oppNewArray.length: " + oppNewArray.length );

   // create new opportunity record in db
   try {
      var oppSaveResult = sforce.connection.create(oppNewArray);
   }
   catch(error) {
      alert("Error saving donation to db. (" + error.faultcode + ") " 
            + error.faultstring );
   }
   //alert("passed new opp rec creation");

   if ( oppSaveResult[0].getBoolean("success") ) {
      //alert("oppSaveResult success detected");
      // get the object that was returned
      oppCreated = oppSaveResult[0];	

      // clone all the contact roles
      var rolesSaveResult = setContactRoles(oppCreated.id, rolesOldArray);

      if (rolesSaveResult[0].getBoolean("success")) {
         // redirect to editing the newly created opportunity
         top.location.replace("/" + oppCreated.id + "/e?retURL=%2F" + oppCreated.id);
      } 				
   } 			

   return "completed";		
}

/*********************************************************************************

* function for cloning all the contact roles for the existing opp to the new opp
**********************************************************************************/
function setContactRoles( newOppId, rolesOldArray){
   //alert("setContactRoles() running");

   // create an array to pass it to update
   var rolesNewArray = new Array();				
   for( var row = 0; row < rolesOldArray.length; row++ ) {
      //alert("processing row " + row );
      var thisRole = rolesOldArray[row];

      // create the object for opportunity contact role
      var role = new sforce.SObject("OpportunityContactRole");

      // link to new opportunity
      role.OpportunityId =  newOppId;

      // set values to match old contact rols
      role.ContactId = thisRole.ContactId;
      role.Role =  thisRole.Role;
      role.IsPrimary = thisRole.IsPrimary;

      // place sobject in array					
      rolesNewArray[row] = role;
   }
   //alert("rolesNewArray.length: " + rolesNewArray.length );
   try {
      var saveResult = sforce.connection.create(rolesNewArray);	
   } catch(error) {
      alert("Error saving contact roles (" + error.faultcode + ") "
            + error.faultstring );
   }
   return saveResult;
}