//*****************************************************************************
//   functions_for_dates.js - JavaScript functions for calculating a new date
//      derived from a specified date with a number of specified months added,
//      and functions for formatting date representations of dates
//   Copyright (C) 2008  Larry Francis Bednar
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or (at your option) any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//   The author may be contacted at:
//      3226 NE 24th Ave, Portland, OR 97212-2442 USA
//      lb@quimdas.com
//      www.quimdas.com
//
//   $Revision: 1.5 $, $Date: 2008/12/12 18:44:47 $
//****************************************************************************

//**************************************************************
function isLeapYr(year) {
   //If the provided year is a leap year, returns "true"
   //   otherwise, returns "
   var mult4 = (year % 4 == 0 );
   var mult100 = ( year % 100 == 0 );
   var mult400 = ( year % 400 == 0 );
   var result = false;
   if ( mult4 ) {
      result = true;
      if ( mult100 && mult400) {
         result = true; 
      } else if (mult100 && !mult400) {
         result = false;
      }
   } 
   return result;
}

//*************************************************************
function dayEquivInMonth(day,month,year) {
   //If the provided value for "day" is larger than
   //   the number of months allowed in the stated
   //   month and year, returns the maximum day allowed.
   //Otherwise, returns the provided "day" value
   //Expects input values with following characteristics:
   //   month in {0,11}
   //   year 
   //   day in {1,31}
   //Converts provided values to integers
   var dayClean = Math.floor(day);
   var monthClean = Math.floor(month);
   var yearClean = Math.floor(year);
   //Array of days in month  
   var MONTH_LIMITS = [31,28,31,30,31,30,31,31,30,31,30,31];   
   dayResult = 0;
   var limit = 0;   //Define local variable
   if ( dayClean >= 1 && dayClean <= 31 &&
        monthClean >= 0 && monthClean <= 11) {
       if (month == 2) {
          //If month is February, check for leap yr
          if ( isLeapYr(year) ) {
             limit = 29;
          } else {
             limit = 28;
          }
      } else {
          limit = MONTH_LIMITS[ month ]; 
      }
      dayResult = ( day > limit? limit: day );
   }
   return dayResult;
}

//**************************************************************
function datePartsAddMonths(day,month,year,monthsToAdd) {
   var addMonths = Math.floor(monthsToAdd);
   //Adds specified number of months to provided date
   //   and returns result as a Date object
   //NOTE: This function considers the addition of
   //   a month to place the result at the end of
   //   the next month, regardless of the number of
   //   days in the months
   //Examples:
   //   addMonths(31,1,2008,1)= 2/29/2008
   //   addMonths(31,1,2007,1)= 2/28/2007
   //   addMonths(28,2,2008,1)= 3/28/2008

   var dateResult = new Date();
   var dayClean = Math.floor(day);
   var monthClean = Math.floor(month) - 1; //convert to (0,11) basis
   var yearClean = Math.floor(year);

   if ( monthsToAdd < 0 ) {
      //If monthsToAdd=-1.5 should imply monthsToAddClean = -1
      var monthsToAddClean = Math.ceil(monthsToAdd);
   } else {
      var monthsToAddClean = Math.floor(monthsToAdd);
   }

   var monthDiff = monthClean + monthsToAddClean; 
   var monthResult = monthDiff - 12 * Math.floor(monthDiff/12);
 
   var yearsToAdd = Math.floor( monthDiff/12);
   var yearResult = yearClean + yearsToAdd;
 
   var dayResult = dayEquivInMonth( dayClean, monthResult, yearResult );

   dateResult.setDate(dayResult);
   dateResult.setMonth(monthResult);    //SetMonth uses {0,11} values
   dateResult.setFullYear(yearResult);
   return dateResult;
}

//*************************************************************
function dateAddMonths(dateStart,months) {
   //Accepts Date object and adds specified number of months,
   //   returns result as a Date object
   var dateResult = datePartsAddMonths( dateStart.getDate(),
                                        dateStart.getMonth()+1,
                                        dateStart.getFullYear(),
                                        months );
   return dateResult;
}

//*************************************************************
function dateToTextMDY( dateIn ) {
   //Accepts Date object, outputs string representing date in
   //   MM/DD/YYYY format
   if (dateIn instanceof Date) {
      var result = (dateIn.getMonth() + 1) + "/"
                 + dateIn.getDate() + "/"
                 + dateIn.getFullYear();
   } else {
      result = "?";
   }
   return result;
}

