//*****************************************************************************
//   functions_for_field_checks.js - JavaScript functions for checking for 
//      presence of speicified fields in specified SalesForce table
//   Copyright (C) 2008  Larry Francis Bednar
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or (at your option) any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//   The author may be contacted at:
//      3226 NE 24th Ave, Portland, OR 97212-2442 USA
//      lb@quimdas.com
//      www.quimdas.com
//
//   $Revision: 1.2 $, $Date: 2008/12/12 18:45:19 $
//****************************************************************************

//****************************************************************
function fieldPresent(tableName, fieldName) {
   // checks for presence of fields in specified SalesForce table
   var output = "";
   tableName = tableName.toLowerCase();
   //alert("table: " + tableName + "\nfieldName: " + fieldName );
   var tableDef = sforce.connection.describeSObjects([tableName]);
   var flds = tableDef[0].fields;
   //alert("flds.length: " + flds.length);
   var result = false;
   for ( var i = 0; i < flds.length; i++ ) {
      var fld = flds[i];
      output += "  " + i + " " + fld.name + "\n"
      if (fld.name == fieldName) {
          result = true;
          break;
      }
   }
   //alert("table fields checked: \n" + output);
   //alert("fieldPresent: " + result);
   return result;
}

//***************************************************************
function missingFields(tableName, fieldNames) {
   // Expects a single value in "tableName" indicating a
   //   SalesForce table, and an array of strings in
   //   "fieldNames" representing fields whose presence
   //   will be checked.  Returns a string
   //   containing names of missing fields
   //alert("missingFields() running");
   //alert("fieldNames.length: " + fieldNames.length);
   var result = "";
   for ( var i = 0; i < fieldNames.length; i++) {
      //alert("checking for field: " + fieldNames[i]);
      if ( fieldPresent(tableName, fieldNames[i]) == false ) {
         result += "  " + fieldNames[i] + "\n";
      }
   }
   //alert("missing fields:\n" + result );
   return result;
}

//***************************************************************
function missingFieldVals(tableName, fieldNames, recId) {
   // Expects a single value in "tableName" indicating a
   //   SalesForce table, and an array of strings in
   //   "fieldNames" representing fields that will be checked
   //   for values, and the Id of the table record to be
   //   checked for values.  Returns a string containing
   //   names of fields having no values assigned
   //alert("missingFieldVals() running");
   //alert("fieldNames.length: " + fieldNames.length);
   var result = "";
   // Translate field names array into string suitable for
   //   required 'retrieve' function field list
   var fieldList = fieldNames.join();
   // Retrieve field values from specified record
   try {
      var qryResult = sforce.connection.retrieve( fieldList, tableName, [recId]);
   } catch(error)  {
      alert("attempt to query for required field values failed");
   }
   alert("passed retrieve attempt");
   //Check for value for each field in list
   var qryResultRows = qryResult[0].getArray("records");
   var qryRow1 = qryResultRows[0];
   for ( var i = 0; i < fieldNames.length; i++ ) {
      alert("checking for value for field: " + fieldNames[i]);
      //If field is empty, boolean interpretation should be 'false'
      if ( qryRow1.getBoolean(fieldNames[i]) == false ) {
         result += "  " + fieldNames[i] + "\n";
      }
   }
   alert("missing fields values:\n" + result );
   return result;
}

